<?php
//
// The command line interface for zphp
//
// Copyright (C) 2009-2016 Väinö Helminen
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//

require_once(__DIR__.'/zdebug.php');

if($argc < 2) {
    echo "Usage: php zcli.php [-v] story.z3\n";
    exit;
}

$savedZ = false;

function hook($state)
{
    global $savedZ;
    if($state) {
        $savedZ = $state;
        return true;
    } else {
        return $savedZ;
    }
}

$COLS = intval(`tput cols`);
$LINES = intval(`tput lines`);

$HOME = $RESTORE = "\n";
$SAVE = $NORMAL = $BOLD = $REV = '';
$ALTSCREEN = '';
$NORMALSCREEN = "\n\n";

if($argv[1] == '-vv') {
    $DEBUG = true;
    $z = new ZDebug($argv[$argc-1]);
} else if ($argv[1] == '-v') {
    $DEBUG = true;
    $z = new ZCheck($argv[$argc-1]);
} else {
    $DEBUG = false;
    $z = new Z($argv[$argc-1]);

    $HOME = `tput home`;
    $SAVE = `tput sc`;
    $RESTORE = `tput rc`;
    $NORMAL = `tput sgr0`;
    $BOLD = `tput bold`;
    $REV = `tput rev`;

    $ALTSCREEN = `tput smcup`;
    if (!$DEBUG && $ALTSCREEN) {
        $NORMALSCREEN = `tput rmcup`;
    } else {
        $ALTSCREEN = str_repeat("\n", $LINES-2);
    }
}

echo "zphp : A Z-machine re-implementation\n\nCopyright (C) 2006-2016 Vaino Helminen\n\n".str_repeat("-", $COLS-1)."\n";
echo $ALTSCREEN;

$stdin = fopen('php://stdin', 'r');
$input = $status = false;
$log = '';
while(true) {
    ob_start();
    $break = $z->run($status, $input, 'hook');
    $output = wordwrap(trim(ob_get_clean()), $COLS);

    if (!$DEBUG) $log .= $output;

    $lines = explode("\n", $output);
    if (count($lines) > $LINES) {
        $from = $to = 0;
        $last = count($lines)-1;
        for($i = 2; $i < $last; ++$i) {
            if (!$lines[$i]) {
                if ($i - $from < $LINES - 2) {
                    $to = $i;
                } else {
                    echo implode("\n", array_slice($lines, 0, $to));
                    echo "\n\n[More]";
                    flush();
                    fgets($stdin);
                    $from = $to + 1;
                    $to = $i;
                }
            }
        }
    }
    echo $output;

    if($break != Z::BREAK_INPUT)
        break;

    if($status) {
        if(count($status) > 1) {
            if ($COLS < 72) {
                $status = sprintf("${BOLD}${REV} %-".($COLS-20)."s ${NORMAL}${REV}Score:%4d / %-4s ${NORMAL}", $status[0], $status[1], $status[2]);
            } else {
                $status = sprintf("${BOLD}${REV} %-".($COLS-27)."s ${NORMAL}${REV}Score:%4d | Moves: %-4s ${NORMAL}", $status[0], $status[1], $status[2]);
            }
        } else {
            $status = sprintf("${BOLD}{REV}%-".($COLS-22)."s ${NORMAL}${REV}%-14s${NORMAL}", $status[0], $status[1]);
        }
        echo "${SAVE}${HOME}${status}${RESTORE}";
    }

    flush();
    $input = fgets($stdin);
    if(feof($stdin))
        break;

    if (!$DEBUG) $log .= $input;
}

echo $NORMALSCREEN;

if (!$DEBUG) echo $log;
